use crate::vga_color as vga;
use bracket_lib::{
    prelude::{to_cp437, BTerm, Rect, RGB},
    random::RandomNumberGenerator,
};

pub const MAP_WIDTH: usize = 64;
pub const MAP_HEIGHT: usize = 23;
const MAP_SIZE: usize = MAP_WIDTH * MAP_HEIGHT;

pub const MAP_X: usize = 1;
pub const MAP_Y: usize = 1;

#[derive(PartialEq, Copy, Clone)]
pub enum TileType {
    Wall,
    Floor,
}

pub struct Map {
    tiles: Vec<TileType>,
    border_fg: RGB,
    border_bg: RGB,
}

impl Map {
    pub fn new() -> Self {
        let mut tiles = vec![TileType::Floor; MAP_SIZE];
        let mut rng = RandomNumberGenerator::new();
        for tile in &mut tiles {
            if rng.roll_dice(1, 4) < 2 {
                *tile = TileType::Wall;
            }
        }
        Self {
            tiles,
            border_fg: RGB::named(vga::get_by_index(rng.range(8, 15))),
            border_bg: RGB::named(vga::get_by_index(rng.range(1, 7) as usize)),
        }
    }

    pub fn draw(&self, ctx: &mut BTerm) {
        // Border
        ctx.fill_region(
            Rect {
                x1: MAP_X as i32 - 1,
                x2: MAP_WIDTH as i32 + MAP_X as i32 + 1,
                y1: MAP_Y as i32 - 1,
                y2: MAP_HEIGHT as i32 + MAP_Y as i32 + 1,
            },
            to_cp437('▓'),
            self.border_fg,
            self.border_bg,
        );

        for (i, tile) in self.tiles.iter().enumerate() {
            let x = i % MAP_WIDTH;
            let y = (i - x) / MAP_WIDTH;

            match tile {
                TileType::Floor => {
                    ctx.set(
                        x + MAP_X,
                        y + MAP_Y,
                        RGB::named(vga::BLACK),
                        RGB::named(vga::BLACK),
                        to_cp437(' '),
                    );
                }
                TileType::Wall => {
                    ctx.set(
                        x + MAP_X,
                        y + MAP_Y,
                        RGB::named(vga::YELLOW),
                        RGB::named(vga::BLACK),
                        to_cp437('▓'),
                    );
                }
            }
        }
    }

    pub fn tile_at(&self, x: usize, y: usize) -> TileType {
        self.tiles[(y * MAP_WIDTH) + x]
    }
}
